#include "HeaderFilter.h"

#include <extensionsystem/pluginmanager.h>
#include <projectexplorer/projectexplorer.h>
#include <projectexplorer/session.h>
#include <projectexplorer/project.h>
#include <filesearch.h>
#include <find/searchresultwindow.h>

#include <QtGui>

struct HeaderFilterData
{
	QFutureWatcher<Utils::FileSearchResultList> watcher;

	ProjectExplorer::ProjectExplorerPlugin* projectExplorer() {
		if(m_projectPlugin) {
			return m_projectPlugin;
		}
        ExtensionSystem::PluginManager* pm = ExtensionSystem::PluginManager::instance();
        m_projectPlugin = pm->getObject<ProjectExplorer::ProjectExplorerPlugin>();
		return m_projectPlugin;
    }

	// Method to search and return the search window
    Find::SearchResultWindow* searchResultWindow() {
		if(m_searchResultWindow) {
			return m_searchResultWindow;
		}
        ExtensionSystem::PluginManager* pm = ExtensionSystem::PluginManager::instance();
        m_searchResultWindow = pm->getObject<Find::SearchResultWindow>();
		return m_searchResultWindow;
    }
private:
    ProjectExplorer::ProjectExplorerPlugin* m_projectPlugin;
	Find::SearchResultWindow *m_searchResultWindow;
};

HeaderFilter::HeaderFilter()
{
    d = new HeaderFilterData;

	// displayResult(int) is called when every a new
	// search result is generated 
    connect(&d->watcher, SIGNAL(resultReadyAt(int)), SLOT(displayResult(int))); 
}

HeaderFilter::~HeaderFilter()
{
    delete d;
}

QString HeaderFilter::id() const
{
    return "HeaderFilter";
}

QString HeaderFilter::displayName() const
{
    return tr("Header Filter");
}

bool HeaderFilter::isEnabled() const
{
	return d->projectExplorer()->session()->projects().count() > 0;
}

bool HeaderFilter::canCancel() const
{
    return true;
}

void HeaderFilter::cancel()
{
    // Do nothing
}

QKeySequence HeaderFilter::defaultShortcut() const
{
    return QKeySequence();
}

QWidget *HeaderFilter::createConfigWidget()
{
    return (new QLabel("This is a header filter"));
}

void HeaderFilter::findAll(const QString &text, Find::FindFlags findFlags) 
{
    // Fetch a list of all open projects
    QList<ProjectExplorer::Project*> projects = d->projectExplorer()->session()->projects();
	// Make a list of files in each project
    QStringList files;
    Q_FOREACH(ProjectExplorer::Project* project, projects) {
        files += project->files(ProjectExplorer::Project::AllFiles);
	}
	// Remove duplicates
    files.removeDuplicates();
	// Search for text in files
	// Variable to hold search results that will 
	// come as the search task progresses
    QFuture<Utils::FileSearchResultList> searchResults;

	// Clear the current search results
    d->searchResultWindow()->clearContents();

	// Begin searching
    QString includeline = "#include <" + text + ">";
    QList<QTextCodec *> codecs;
	codecs << QTextCodec::codecForLocale();
	searchResults = Utils::findInFiles(includeline,
		new Utils::FileIterator(files, codecs),
		Find::textDocumentFlagsForFindFlags(findFlags));
	// Let the watcher monitor the search results
    d->watcher.setFuture(searchResults);
}

void HeaderFilter::displayResult(int index)
{
	Utils::FileSearchResultList results = d->watcher.future().resultAt(index);
	Q_FOREACH(Utils::FileSearchResult result, results) {
		d->searchResultWindow()->addResult(result.fileName, 
			result.lineNumber, 
			result.matchingLine, 
			result.matchStart, 
			result.matchLength);
	}
}
